from  tests.integration import asserts
from tests.integration.conftest import account


def test_account_user_can_be_created(account_user, account_user_params):
    asserts.assert_resource(account_user)
    asserts.assert_resource_params(account_user, account_user_params)


def test_account_users_list(api,account, account_user):
    users = account.users.list()
    assert len(users) > 1


def test_account_users_get_by_id(account, account_user, account_user_params):
    account_user = account.users.read(account_user['id'])
    asserts.assert_resource(account_user)
    asserts.assert_resource_params(account_user, account_user_params)


def test_account_user_can_be_updated(account_user, account_user_update_params):
    updated_user = account_user.update(account_user_update_params)
    asserts.assert_resource(updated_user)
    asserts.assert_resource_params(updated_user, account_user_update_params)


def test_account_user_change_status(account_user):
    assert account_user['state'] == 'pending'
    updated_account_user = account_user.activate()
    assert updated_account_user['state'] == 'active'

    updated_account_user = account_user.suspend()
    assert updated_account_user['state'] == 'suspended'

    updated_account_user = account_user.un_suspend()
    assert updated_account_user['state'] == 'active'


def test_account_user_change_role(account_user):
    assert account_user['role'] == 'member'

    updated_account_user = account_user.set_as_admin()
    assert  updated_account_user['role'] == 'admin'

    updated_account_user = account_user.set_as_member()
    assert updated_account_user['role'] == 'member'



import pytest

from threescale_api.errors import ApiClientError

from tests.integration import asserts

def test_list_methods(backend_hits_metric, backend_method):
    assert len(backend_hits_metric.methods.list()) >= 1

def test_should_create_method(backend_method, method_params):
    asserts.assert_resource(backend_method)
    asserts.assert_resource_params(backend_method, method_params)


def test_should_not_create_method_for_custom_metric(backend_hits_metric, method_params):
    resource = backend_hits_metric.methods.create(params=method_params, throws=False)
    asserts.assert_errors_contains(resource, ['system_name'])


def test_should_friendly_name_be_required(backend_hits_metric):
    resource = backend_hits_metric.methods.create(params={}, throws=False)
    asserts.assert_errors_contains(resource, ['friendly_name'])


def test_should_raise_api_exception(backend_hits_metric):
    with pytest.raises(ApiClientError):
        backend_hits_metric.methods.create(params={})


def test_should_read_method(backend_method, method_params):
    resource = backend_method.read()
    asserts.assert_resource(resource)
    asserts.assert_resource_params(resource, method_params)


def test_should_update_method(backend_method, updated_method_params):
    resource = backend_method.update(params=updated_method_params)
    asserts.assert_resource(resource)
    asserts.assert_resource_params(resource, updated_method_params)


def test_should_delete_method(backend_hits_metric, updated_method_params):
    resource = backend_hits_metric.methods.create(params=updated_method_params)
    assert resource.exists()
    resource.delete()
    assert not resource.exists()


def test_should_list_methods(backend_hits_metric):
    resources = backend_hits_metric.methods.list()
    assert len(resources) == 1



import pytest

from threescale_api import client


@pytest.fixture()
def url():
    return 'http://localhost'


@pytest.fixture()
def token():
    return 'test-token'


def _create_client(url, token, **kwargs) -> client.ThreeScaleClient:
    return client.ThreeScaleClient(url=url, token=token, **kwargs)


@pytest.fixture()
def api(url, token):
    return _create_client(url, token)


@pytest.mark.smoke
def test_api_client_initialization(api, url):
    assert api.url == url
    assert api.parent == api
    assert api.threescale_client == api
    assert api.admin_api_url == f'{url}/admin/api'



from threescale_api.utils import request2curl


def test_request2curl():
    URL = "http://example.invalid"
    HEADERS = {"X-Header": "this"}
    HEADERS_STR = "-H 'X-Header: this'"
    DATA = {'key': 'value'}
    BODY_STR = "-d key=value"

    request = _Request("GET", URL, None, None)
    assert request2curl(request) == f"curl -X GET {URL}"

    request = _Request("GET", URL, HEADERS, None)
    assert request2curl(request) == f"curl -X GET {HEADERS_STR} {URL}"

    request = _Request("POST", URL, HEADERS, DATA)
    assert request2curl(request) == f"curl -X POST {HEADERS_STR} {BODY_STR} {URL}"

    request = _Request("PUT", URL, None, DATA, True)
    assert request2curl(request) == f"curl -X PUT {BODY_STR} {URL}"


class _Request:
    def __init__(self, method, url, headers, data, encode=False):
        self.method = method
        self.url = url
        self.headers = headers
        self.body = None
        if data:
            self.body = "&".join([f"{key}={value}" for key, value in data.items()])
        if encode and self.body:
            self.body = self.body.encode("utf-8")



from tests.integration import asserts

def test_list_service_plans(service):
    resource = service.service_plans.list()
    assert len(resource) >= 1

def test_check_service_plan_creation(service_plan, service_plan_params):
    asserts.assert_resource(service_plan)
    asserts.assert_resource_params(service_plan, service_plan_params)
    assert service_plan.exists()

def test_read_service_plans(service, service_plan, service_plan_params):
    asserts.assert_resource(service_plan)
    resource = service.service_plans.read(service_plan.entity_id)
    asserts.assert_resource(resource)
    asserts.assert_resource_params(service_plan,service_plan_params)

def test_update_service_plans(service, service_plan, service_plan_params):
    asserts.assert_resource(service_plan)
    service_plan['state'] = 'publish'
    service_plan['approval_required'] = True
    resource = service.service_plans.update(service_plan.entity_id, service_plan_params)
    asserts.assert_resource(resource)
    asserts.assert_resource_params(service_plan, service_plan_params)

def test_set_default_service_plan(service_plan, service_plan_params):
    asserts.assert_resource(service_plan)
    resource = service_plan.set_default()
    asserts.assert_resource(resource)
    asserts.assert_resource_params(service_plan, service_plan_params)



from tests.integration import asserts

def test_list_service_subscriptions(account):
    resource = account.service_subscriptions.list()
    assert len(resource) >= 1

def test_create_service_subscription(service_subscription, service_subscription_params):
    asserts.assert_resource(service_subscription)
    asserts.assert_resource_params(service_subscription, service_subscription_params)

def test_read_service_subscription(account, service_subscription, service_subscription_params):
    resource = account.service_subscriptions.read(service_subscription.entity_id)
    asserts.assert_resource(resource)
    asserts.assert_resource_params(service_subscription,service_subscription_params)

def test_change_plan_service_subscription(account, account_plan, service_plan,
                                          service_subscription):
    asserts.assert_resource(account)
    asserts.assert_resource(account_plan)
    asserts.assert_resource(service_plan)
    account.service_subscriptions.change_plan(service_subscription.entity_id,
                                                         service_plan.entity_id)

def test_approve_service_subscription(account, service_subscription, service_plan):
    asserts.assert_resource(service_subscription)
    asserts.assert_resource(account)
    asserts.assert_resource(service_plan)
    resource = service_subscription.approve()
    asserts.assert_resource(resource)



from tests.integration import asserts


def test_accounts_list(api, account):
    accounts = api.accounts.list()
    assert len(accounts) >= 1


def test_account_can_be_created(api, account, account_params):
    asserts.assert_resource(account)
    asserts.assert_resource_params(account, account_params, ['org_name'])


def test_account_can_be_read(api, account, account_params):
    read = api.accounts.read(account.entity_id)
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, account_params, ['org_name'])


def test_account_can_be_read_by_name(api, account, account_params):
    account_name = account['org_name']
    read = api.accounts[account_name]
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, account_params, ['org_name'])


def test_account_can_be_find_by_name(api, account_user, account_params):
    account2 = api.accounts.find(dict(username=account_user['username']))
    asserts.assert_resource(account2)
    asserts.assert_resource_params(account2, account_params, ['org_name'])


def test_account_update(api,account, update_account_params):
    updated_account = account.update(params=update_account_params)
    asserts.assert_resource(updated_account)
    asserts.assert_resource_params(updated_account, update_account_params)


def test_get_account_plan(account, account_get_plan_params):
    plan =  account.get_account_plan()
    asserts.assert_resource(plan)
    asserts.assert_resource_params(plan,account_get_plan_params, ['name'])


def test_users_list(api, account):
    assert len(account.users.list()) >= 1



from .asserts import assert_resource
from .conftest import get_suffix


def test_provider_account_read(provider_account):
    assert_resource(provider_account)


def test_provider_account_update(provider_account, api):
    update_access_code = get_suffix()
    api.provider_accounts.update(dict(site_access_code=update_access_code))
    update = api.provider_accounts.fetch()
    assert update['site_access_code'] == update_access_code


def test_provider_account_resource_update(provider_account, api):
    update_access_code = get_suffix()
    provider_account.update(dict(site_access_code=update_access_code))
    assert provider_account['site_access_code'] == update_access_code
    update = api.provider_accounts.fetch()
    assert update['site_access_code'] == update_access_code



from tests.integration import asserts


def test_active_docs_can_be_created(active_doc, active_docs_params):
    asserts.assert_resource(active_doc)
    asserts.assert_resource_params(active_doc, active_docs_params)


def test_active_docs_fetch(active_doc):
    ac = active_doc.client.fetch(int(active_doc['id']))
    assert ac
    assert ac['id'] == active_doc['id']


def test_active_docs_list(api, active_doc):
    active_docs = api.active_docs.list()
    assert len(active_docs) >= 1


def test_active_docs_update(active_doc, active_docs_update_params):
    updated_active_doc = active_doc.update(params=active_docs_update_params)
    asserts.assert_resource(updated_active_doc)
    asserts.assert_resource_params(updated_active_doc, active_docs_update_params)



from tests.integration import asserts


def test_account_plans_list(api, account_plan):
    account_plans = api.account_plans.list()
    assert len(account_plans) >= 1


def test_account_plan_can_be_created(api, account_plan, account_plans_params):
    asserts.assert_resource(account_plan)
    asserts.assert_resource_params(account_plan, account_plans_params)

def test_account_plan_can_be_read(api, account_plan, account_plans_params):
    read = api.account_plans.read(account_plan.entity_id)
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, account_plans_params)


def test_account_plan_update(account_plan, account_plans_update_params):
    updated_account_plan = account_plan.update(params=account_plans_update_params)
    asserts.assert_resource(updated_account_plan)
    asserts.assert_resource_params(updated_account_plan, account_plans_update_params)




""" Test CMS API """

import pytest
from tests.integration import asserts
from threescale_api import errors
from .asserts import assert_resource, assert_resource_params


# Files
def test_file_list(api, cms_file):
    """ List all files. """
    assert len(api.cms_files.list()) >= 1


def test_file_can_be_created(cms_file_data, cms_file):
    """ Is file created properly? """
    assert_resource(cms_file)
    assert_resource_params(cms_file, cms_file_data)


def test_file_can_be_read(api, cms_file_data, cms_file):
    """ It is possible to get file by ID? """
    read = api.cms_files.read(cms_file.entity_id)
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, cms_file_data)


def test_file_can_be_read_by_name(api, cms_file_data, cms_file):
    """ It is possible to get file by name? """
    file_path = cms_file['path']
    read = api.cms_files[file_path]
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, cms_file_data)


def test_file_can_be_updated(cms_file_data, cms_file):
    """ Can be file object updated? """
    updated_path = cms_file['path'] + 'up'
    cms_file['path'] = cms_file['path'] + 'up'
    # https://issues.redhat.com/browse/THREESCALE-9571
    for item in "created_at", "updated_at", "url", "title", "content_type":
        cms_file.pop(item)
    cms_file.update()
    assert cms_file['path'] == updated_path
    updated = cms_file.read()
    assert updated['path'] == updated_path
    assert cms_file['path'] == updated_path


# Sections


def test_section_list(api, cms_section):
    """ List all sections. """
    assert len(api.cms_sections.list()) >= 1


def test_section_can_be_created(cms_section_params, cms_section):
    """ Is section created properly? """
    assert_resource(cms_section)
    assert_resource_params(cms_section, cms_section_params)


def test_section_can_be_read(api, cms_section_params, cms_section):
    """ It is possible to get section by ID? """
    read = api.cms_sections.read(cms_section.entity_id)
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, cms_section_params)


def test_section_can_be_updated(cms_section_params, cms_section):
    """ Can be section object updated? """
    updated_title = cms_section['title'] + 'up'
    cms_section['title'] = cms_section['title'] + 'up'
    # https://issues.redhat.com/browse/THREESCALE-9571
    for item in "created_at", "updated_at":
        cms_section.pop(item)
    cms_section.update()
    assert cms_section['title'] == updated_title
    updated = cms_section.read()
    assert updated['title'] == updated_title
    assert cms_section['title'] == updated_title


def test_builtin_section_delete(api):
    """It is not possible to delete section partial."""
    with pytest.raises(errors.ApiClientError) as exc_info:
        api.cms_sections.list()[0].delete()
    assert exc_info.value.code == 422


# Partials
# builtin


def test_builtin_partials_list(api):
    """ List all sections. """
    assert len(api.cms_builtin_partials.list()) >= 1


def test_builtin_partial_can_be_read(api):
    """ It is possible to get partial by ID? """
    cms_partial = api.cms_builtin_partials.list()[-1]
    read = api.cms_builtin_partials.read(cms_partial.entity_id)
    asserts.assert_resource(read)

def test_builtin_partial_delete(api):
    """It is not possible to delete builtin partial."""
    with pytest.raises(errors.ApiClientError) as exc_info:
        api.cms_builtin_partials.list()[0].delete()
    assert exc_info.value.code == 422

# user


def test_partial_list(api, cms_partial):
    """ List all user defined partials. """
    parts_list = api.cms_partials.list()
    assert len(parts_list) >= 1
    assert all('draft' in part.entity.keys() and 'published' in part.entity.keys()
                for part in parts_list)


def test_partial_can_be_created(cms_partial_params, cms_partial):
    """ Is partial created properly? """
    assert_resource(cms_partial)
    assert_resource_params(cms_partial, cms_partial_params)


def test_partial_can_be_read(api, cms_partial_params, cms_partial):
    """ It is possible to get partial by ID? """
    read = api.cms_partials.read(cms_partial.entity_id)
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, cms_partial_params)


def test_partial_can_be_updated(cms_partial_params, cms_partial):
    """ Can be partial object updated? """
    updated_draft = cms_partial['draft'] + 'up'
    cms_partial['draft'] = cms_partial['draft'] + 'up'
    # https://issues.redhat.com/browse/THREESCALE-9571
    for item in "created_at", "updated_at", "published":
        cms_partial.pop(item)
    cms_partial.update()
    assert cms_partial['draft'] == updated_draft
    updated = cms_partial.read()
    assert updated['draft'] == updated_draft
    assert cms_partial['draft'] == updated_draft


def test_partial_publish(cms_partial):
    """ Test publishing of partials. """
    assert cms_partial.entity.get('published', None) is None
    draft = cms_partial['draft']
    cms_partial = cms_partial.publish()
    assert cms_partial['draft'] == None
    assert draft == cms_partial['published']


# Pages
# builtin


def test_builtin_pages_list(api):
    """ List all sections. """
    assert len(api.cms_builtin_pages.list()) >= 1


def test_builtin_page_can_be_read(api):
    """ It is possible to get page by ID? """
    cms_page = api.cms_builtin_pages.list()[-1]
    read = api.cms_builtin_pages.read(cms_page.entity_id)
    asserts.assert_resource(read)


def test_builtin_page_delete(api):
    """It is not possible to delete builtin page."""
    with pytest.raises(errors.ApiClientError) as exc_info:
        api.cms_builtin_pages.list()[0].delete()
    assert exc_info.value.code == 422


# user


def test_page_list(api, cms_page):
    """ List all user defined pages. """
    assert len(api.cms_pages.list()) >= 1


def test_page_can_be_created(cms_page_params, cms_page):
    """ Is page created properly? """
    assert_resource(cms_page)
    assert_resource_params(cms_page, cms_page_params)


def test_page_can_be_read(api, cms_page_params, cms_page):
    """ It is possible to get page by ID? """
    read = api.cms_pages.read(cms_page.entity_id)
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, cms_page_params)


def test_page_can_be_updated(cms_page_params, cms_page):
    """ Can be page object updated? """
    updated_draft = cms_page['draft'] + 'up'
    cms_page['draft'] = cms_page['draft'] + 'up'
    # https://issues.redhat.com/browse/THREESCALE-9571
    for item in "created_at", "updated_at", "hidden", "published":
        cms_page.pop(item)
    cms_page.update()
    assert cms_page['draft'] == updated_draft
    updated = cms_page.read()
    assert updated['draft'] == updated_draft
    assert cms_page['draft'] == updated_draft


def test_page_publish(cms_page):
    """ Test publishing of pages. """
    assert cms_page.entity.get('published', None) is None
    draft = cms_page['draft']
    cms_page = cms_page.publish()
    assert draft == cms_page['published']


# Layouts


def test_layout_list(api, cms_layout):
    """ List all user defined layouts. """
    assert len(api.cms_layouts.list()) >= 1


def test_layout_can_be_created(cms_layout_params, cms_layout):
    """ Is layout created properly? """
    assert_resource(cms_layout)
    assert_resource_params(cms_layout, cms_layout_params)


def test_layout_can_be_read(api, cms_layout_params, cms_layout):
    """ It is possible to get layout by ID? """
    read = api.cms_layouts.read(cms_layout.entity_id)
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, cms_layout_params)


def test_layout_can_be_updated(cms_layout_params, cms_layout):
    """ Can be layout object updated? """
    updated_draft = cms_layout['draft'] + 'up'
    cms_layout['draft'] = cms_layout['draft'] + 'up'
    # https://issues.redhat.com/browse/THREESCALE-9571
    for item in "created_at", "updated_at", "published":
        cms_layout.pop(item)
    cms_layout.update()
    assert cms_layout['draft'] == updated_draft
    updated = cms_layout.read()
    assert updated['draft'] == updated_draft
    assert cms_layout['draft'] == updated_draft


def test_layout_publish(cms_layout):
    """ Test publishing of layouts. """
    assert cms_layout.entity.get('published', None) is None
    draft = cms_layout['draft']
    cms_layout = cms_layout.publish()
    assert draft == cms_layout['published']

# filters
def test_section_filter(api, cms_section):
    """ Test section filtering """
    assert all(sec['parent_id'] == cms_section['parent_id']
                for sec in api.cms_sections.select_by(parent_id=cms_section['parent_id']))
    assert api.cms_sections.select_by(title=cms_section['title'])[0] == cms_section

def test_files_filter(api, cms_file, cms_section):
    """ Test files filtering """
    assert api.cms_files.select_by(section_id=cms_section['id'])[0] == cms_file
    assert api.cms_files.select_by(path=cms_file['path'])[0] == cms_file

# https://issues.redhat.com/browse/THREESCALE-9191?focusedId=22406548&page=com.atlassian.jira.plugin.system.issuetabpanels:comment-tabpanel#comment-22406548
def test_partial_filter(api, cms_partial, cms_section):
    """ Test partial filtering """
    assert api.cms_partials.select_by(section_id=cms_section['id']) == []
    assert api.cms_partials.select_by(system_name=cms_partial['system_name'])[0] == cms_partial

def test_layout_filter(api, cms_layout, cms_section):
    """ Test layout filtering """
    assert api.cms_layouts.select_by(section_id=cms_section['id']) == []
    assert api.cms_layouts.select_by(title=cms_layout['title'])[0] == cms_layout

def test_page_filter(api, cms_section, cms_page):
    """ Test page filtering """
    assert api.cms_pages.select_by(section_id=cms_section['id'])[0] == cms_page
    assert api.cms_pages.select_by(title=cms_page['title'])[0] == cms_page



from tests.integration import asserts
from tests.integration.conftest import backend


def test_backend_usage_can_be_created(backend_usage, backend_usage_params):
    asserts.assert_resource(backend_usage)
    asserts.assert_resource_params(backend_usage, backend_usage_params, [param for param in backend_usage_params.keys() if param != 'backend_api_id'])


def test_backend_usages_list(api, backend_usage, backend):
    assert  len(backend.usages()) >= 1


def test_backend_usage_update(backend_usage, backend, backend_usage_update_params):
    updated_backend_usage = backend_usage.update(backend_usage_update_params)
    asserts.assert_resource(updated_backend_usage)
    asserts.assert_resource_params(updated_backend_usage, backend_usage_update_params, [param for param in backend_usage_update_params.keys() if param != 'backend_api_id'])



import random
import string
import base64
import secrets
import pytest

from tests.integration import asserts


@pytest.fixture(scope='module')
def update_application_params():
    suffix = secrets.token_urlsafe(8)
    name = f"updated-{suffix}"
    return dict(name=name, description=name)


def test_application_can_be_created(application, application_params):
    asserts.assert_resource(application)
    asserts.assert_resource_params(application, application_params)


def test_application_list(account, application):
    applications = account.applications.list()
    assert len(applications) > 0


def test_application_update(application, update_application_params):
    updated_application = application.update(params=update_application_params)
    asserts.assert_resource(updated_application)
    asserts.assert_resource_params(updated_application, update_application_params)


def test_application_key_can_be_created(app_key, app_key_params):
    asserts.assert_resource(app_key)
    assert app_key["value"] == app_key_params["key"]


def test_application_key_list(application, app_key):
    keys = application.keys.list()
    assert len(keys) > 0


def test_application_update_userkey(application):
    new_key = "".join(random.choices(string.ascii_letters + string.digits + "-_.", k=255))
    updated_application = application.update(params={"user_key": new_key})
    asserts.assert_resource(updated_application)
    assert updated_application["user_key"] == new_key


def test_application_update_userkey_base64(application):
    new_key = "".join(random.choices(string.ascii_letters + string.digits + "-_.", k=41))
    new_key = base64.b64encode(new_key.encode('ascii')).decode()
    updated_application = application.update(params={"user_key": new_key})
    asserts.assert_resource(updated_application)
    assert updated_application["user_key"] == new_key



import pytest
import responses

from threescale_api import client, errors


@pytest.fixture()
def url():
    return 'http://localhost'


@pytest.fixture()
def token():
    return 'test-token'


@pytest.fixture()
def access_token(token):
    return dict(access_token=token)


@pytest.fixture()
def rest(url, token):
    return client.RestApiClient(url=url, token=token)


@pytest.mark.smoke
def test_rest_api_initialization(rest, url):
    assert rest.url == url


@responses.activate
@pytest.mark.smoke
def test_404_response(rest, url, token):
    responses.add(responses.GET, f'{url}.json',
                  json={'error': 'not found'}, status=404)

    with pytest.raises(errors.ApiClientError):
        rest.request()


@pytest.mark.smoke
@responses.activate
def test_get_response(rest, url, token):
    add_response(responses.GET, url)

    resp = rest.get(path='/some')
    assert_valid_message(resp=resp, url=url, token=token, method='GET')


@pytest.mark.smoke
@responses.activate
def test_delete_response(rest, url, token):
    add_response(responses.DELETE, url)

    resp = rest.delete(path='/some')
    assert_valid_message(resp=resp, url=url, token=token, method='DELETE')


@pytest.mark.smoke
@responses.activate
def test_post_response(rest, url, token):
    add_response(responses.POST, url)

    resp = rest.post(path='/some', json={'bar': 'foo'})
    assert_valid_message(resp=resp, url=url, token=token, method='POST', body=True)


@pytest.mark.smoke
@responses.activate
def test_put_response(rest, url, token):
    add_response(responses.PUT, url)

    resp = rest.put(path='/some', json={'bar': 'foo'})
    assert_valid_message(resp=resp, url=url, token=token, method='PUT', body=True)


@pytest.mark.smoke
@responses.activate
def test_patch_response(rest, url, token):
    add_response(responses.PATCH, url)
    resp = rest.patch(path='/some', json={'bar': 'foo'})
    assert_valid_message(resp=resp, url=url, token=token, method='PATCH', body=True)


def add_response(method, url):
    responses.add(method, f'{url}/some.json', json=dict(message='what ever'), status=200)


def assert_valid_message(resp, url, method, token, body=False):
    assert resp.ok
    assert resp.status_code == 200
    assert resp.json()['message'] == 'what ever'
    request = responses.calls[0].request
    assert request.url == f'{url}/some.json?access_token={token}'
    assert request.path_url == f'/some.json?access_token={token}'
    if body:
        assert request.body.decode(encoding='utf-8') == '{"bar": "foo"}'
    assert request.method == method



import logging
from logging.config import dictConfig


def _to_log_level_map(log_map: dict) -> dict:
    result = {}
    for level, arr in log_map.items():
        for item in arr:
            result[item] = level
    return result


_LOG_LEVEL_MAP_DEFINITION = {
    logging.CRITICAL: ["critical", "c", "crit"],
    logging.ERROR: ["error", "e", "err"],
    logging.WARNING: ["warning", "w", "warn"],
    logging.INFO: ["info", "i", "inf"],
    logging.DEBUG: ["debug", "d", "dbg"],
    logging.NOTSET: ["notset", "n", "nst"],
}

LOG_LEVEL_MAP = _to_log_level_map(_LOG_LEVEL_MAP_DEFINITION)


def to_log_level(level: str, default=logging.NOTSET) -> int:
    if isinstance(level, int):
        return level

    if level is not None:
        return LOG_LEVEL_MAP.get(level.lower(), default)
    return default


def load_config(level: str = "INFO", handler_level=None, api_level=None, tests_level=None):
    config = dict(
        version=1,
        formatters={"verbose": {"format": "%(asctime)s %(name)-12s %(levelname)-8s %(message)s"}},
        handlers={
            "console": {
                "class": "logging.StreamHandler",
                "formatter": "verbose",
                "level": to_log_level(handler_level, default=level),
            }
        },
        loggers={
            "threescale_api": {"handlers": ["console"], "level": to_log_level(api_level, default=level)},
            "tests": {"handlers": ["console"], "level": to_log_level(handler_level, default=level)},
        },
    )

    dictConfig(config)
